package org.bdware.client.test;

import org.bouncycastle.crypto.CryptoException;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.params.ECPublicKeyParameters;
import org.bouncycastle.math.ec.ECPoint;
import org.bouncycastle.pqc.math.linearalgebra.ByteUtils;
import org.junit.Test;
import org.zz.gmhelper.BCECUtil;
import org.zz.gmhelper.SM2KeyPair;
import org.zz.gmhelper.SM2Util;

import java.security.SecureRandom;
import java.util.Random;

public class SM2Test {
    public static String strSign = "helloworld";

    private static byte[] str2Byte(String str) {
        byte[] ret = new byte[str.length() / 2];
        for (int i = 0; i < str.length() / 2; i++) {
            int val = (char2Int(str.charAt(i * 2)) << 4) + (char2Int(str.charAt(i * 2 + 1)));
            ret[i] = (byte) (val & 0xff);
        }
        // TODO Auto-generated method stub
        return ret;
    }

    private static int char2Int(char c) {
        int ret = c;
        if (c >= 'a' && c <= 'f') {
            ret = '0' + 10 + c - 'a';
        }
        ret = ret - '0';
        if (ret >= 0 && ret < 16) {
            return ret;
        } else
            throw new IllegalArgumentException(
                    "the input should be 0-9 and a-f, but received " + c);
    }

    private static String bytes2Str(byte[] result) {
        StringBuilder sb = new StringBuilder();
        for (byte b : result) {
            sb.append(String.format("%x%x", (b >> 4) & 0xf, b & 0xf));
        }
        return sb.toString();
    }

    @Test
    public void generateKeyPair() {
        SM2KeyPair pair = SM2Util.generateSM2KeyPair();
        System.out.println("[KeyPair] " + pair.toJson());
    }

    @Test
    public void loadKeyAndDoSignature() {
        try {
            String keyStr =
                    "{\"publicKey\":\"04bbc40ff490a71a1fe2e8d76b49dd73ef3b19fd90b5cf2fc4a9b2f02bdb3c55a58f32ded81c508fc894dac32e0afe5703879c059ed2e6999f17d5a4172549f0b4\",\"privateKey\":\"cc62de3037d0944854c6f3fabea078b1e92c5c2404f5421e929827a481ac4cc5\"}";
            SM2KeyPair pair = SM2KeyPair.fromJson(keyStr);
            String signature =
                    ByteUtils.toHexString(
                            SM2Util.sign(pair.getPrivateKeyParameter(), strSign.getBytes()));
            System.out.println("[Signature]" + signature);

        } catch (CryptoException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void verifySignature() {
        String signature =
                "3046022100ced6cab1c3f93b8768f14a5f69d493ba54619221375897613b6182dfd0fe2a6b02210091454953b42d8c5e07d2552b5d02c6df04d35c5c09703df41d7043953c38f71b";
        String pubKeyStr =
                "04bbc40ff490a71a1fe2e8d76b49dd73ef3b19fd90b5cf2fc4a9b2f02bdb3c55a58f32ded81c508fc894dac32e0afe5703879c059ed2e6999f17d5a4172549f0b4";
        ECPublicKeyParameters pubKey =
                BCECUtil.createECPublicKeyFromStrParameters(
                        pubKeyStr, SM2Util.CURVE, SM2Util.DOMAIN_PARAMS);
        boolean b = SM2Util.plainStrVerify(pubKeyStr, strSign, signature);
        System.out.println("[verifySignature] verify result:" + b);
    }

    String randStr(int len) {
        StringBuilder sb = new StringBuilder();
        Random r = new SecureRandom();
        for (int i = 0; i < len; i++) {
            sb.append(Math.abs(r.nextInt() % 10));
        }
        return sb.toString();
    }

    @Test
    public void encrypt() {
        String plain = randStr(20);
        String key =
                "{\"privateKey\":\"e45f3a594862ae71ffff827853eda0096bf643e5975224f2e2166507e70af444\",\"publicKey\":\"04337f32fe57b0f1b6f6fd729f98aad9c816770bcca1173857df337c4c8c4f96508d69f9a46f81cb429079a9b82b6232aca286b6bfe6bafb358dc4245cfdeb070e\"}";
        SM2KeyPair pair = SM2KeyPair.fromJson(key);
        try {
            byte[] encrypted = SM2Util.encrypt(pair.getPublicKey(), plain.getBytes());
            System.out.println("encryped:" + bytes2Str(encrypted));
            String origin = new String(SM2Util.decrypt(pair.getPrivateKeyParameter(), encrypted));
            System.out.println("decrypted:" + origin);
            assert origin.equals(plain);
        } catch (InvalidCipherTextException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void decryptFromJS() throws Exception {
        String key =
                "{\"privateKey\":\"e45f3a594862ae71ffff827853eda0096bf643e5975224f2e2166507e70af444\",\"publicKey\":\"04337f32fe57b0f1b6f6fd729f98aad9c816770bcca1173857df337c4c8c4f96508d69f9a46f81cb429079a9b82b6232aca286b6bfe6bafb358dc4245cfdeb070e\"}";
        SM2KeyPair pair = SM2KeyPair.fromJson(key);
        String encrypted =
                "fa68b741a21e5cb63e9bffab8e1bfa8af6b89172fa0b656c2e557df6dac5858a7a0f47273a1d457421f76510d872db9f0a3e7bd1f294c3fe0e8e524f6bf4957f3a930c2c83980593cb3875421e03116adb4c6106522d0c366affd430b8371fbc6818a7";
        String str =
                new String(
                        org.zz.gmhelper.SM2Util.decrypt(
                                pair.getPrivateKeyParameter(), ByteUtils.fromHexString(encrypted)));
        System.out.println(str);
    }

    @Test
    public void importPubkey() {
        String pubKeyStr =
                "04bbc40ff490a71a1fe2e8d76b49dd73ef3b19fd90b5cf2fc4a9b2f02bdb3c55a58f32ded81c508fc894dac32e0afe5703879c059ed2e6999f17d5a4172549f0b4";
        byte[] pubKeyBytes = str2Byte(pubKeyStr);
        ECPoint pubKey = SM2KeyPair.publicKeyStr2ECPoint(pubKeyStr).getQ();
    }
}
