package org.bdware.client.test;

import com.google.gson.JsonObject;
import org.bdware.client.ContractRequest;
import org.bdware.client.ResultCallback;
import org.bdware.client.SmartContractClient;
import org.bdware.client.ws.Action;
import org.junit.BeforeClass;
import org.junit.Test;

public class SmartClientTest {

    private static SmartContractClient testClient = getTestClient();
    static boolean isLogin = false;

    public static SmartContractClient getTestClient() {
        return new SmartContractClient("ws://022.node.internetapi.cn:21030/SCIDE/SCExecutor") {
            @Override
            public void onLogin(JsonObject jo) {
                isLogin = true;
                System.out.println("[SmartClientTest] --> login:" + jo.toString());
            }
            // !!!!!Important!!!!
            // if you want to handle onXXX message, just use @Action annotation
            // there are more onXXX actions described in
            // https://public.internetapi.cn/docs/bdcontract/doc/ContractAPI.html#id194
            @Action
            public void onHashResult(JsonObject jo) {
                System.out.println("[SmartClientTest] --> onHashResult:" + jo.toString());
            }
        };
    }

    @BeforeClass
    public static void connect() {
        System.out.println("WaitForConnect");
        testClient.waitForConnect();
        testClient.login();
        for (; !isLogin; ) Thread.yield();
        testClient.listContractProcess();
        System.out.println("Connected!");
    }

    @Test
    public void executeContractSync() throws InterruptedException {
        ContractRequest cr = new ContractRequest();
        cr.setContractID("ScoreCollector");
        cr.setAction("getAllScore");
        // use the sm2keypair of the connection to do sign automatically by default.
        // can use cr.doSignature(...); to sign in other sm2keypairs
        JsonObject result = testClient.executeContractSync(cr);
        System.out.println(result.toString());
        Thread.sleep(3000);
    }

    @Test
    public void executeContractAsync() {
        ContractRequest cr = new ContractRequest();
        cr.setContractID("ScoreCollector");
        cr.setAction("getAllScore");
        testClient.executeContract(
                cr,
                new ResultCallback() {
                    @Override
                    public void onResult(JsonObject r) {
                        System.out.println("[Async] --> " + r.toString());
                        synchronized (cr) {
                            cr.notify();
                        }
                    }
                });
        synchronized (cr) {
            try {
                cr.wait();
                Thread.sleep(10000);

            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
